@@ -21,13 +21,15 @@
 
 """A GIT branch and repository format implementation for bzr."""
 
+from __future__ import absolute_import
+
 import os
 import sys
 
 import bzrlib
 import bzrlib.api
 
-from info import (
+from bzrlib.plugins.git.info import (
     bzr_compatible_versions,
     bzr_plugin_version as version_info,
     dulwich_minimum_version,
@@ -121,13 +123,9 @@ from bzrlib.revisionspec import revspec_registry
 revspec_registry.register_lazy("git:", "bzrlib.plugins.git.revspec",
     "RevisionSpec_git")
 
-from bzrlib.revisionspec import dwim_revspecs, RevisionSpec_dwim
-if getattr(RevisionSpec_dwim, "append_possible_lazy_revspec", None):
-    RevisionSpec_dwim.append_possible_lazy_revspec(
-        "bzrlib.plugins.git.revspec", "RevisionSpec_git")
-else: # bzr < 2.4
-    from bzrlib.plugins.git.revspec import RevisionSpec_git
-    dwim_revspecs.append(RevisionSpec_git)
+from bzrlib.revisionspec import RevisionSpec_dwim
+RevisionSpec_dwim.append_possible_lazy_revspec(
+    "bzrlib.plugins.git.revspec", "RevisionSpec_git")
 
 
 class LocalGitProber(Prober):
@@ -170,7 +168,8 @@ class RemoteGitProber(Prober):
 
     def probe_http_transport(self, transport):
         from bzrlib import urlutils
-        url = urlutils.join(transport.external_url(), "info/refs") + "?service=git-upload-pack"
+        base_url, _ = urlutils.split_segment_parameters(transport.external_url())
+        url = urlutils.join(base_url, "info/refs") + "?service=git-upload-pack"
         from bzrlib.transport.http._urllib import HttpTransport_urllib, Request
         if isinstance(transport, HttpTransport_urllib):
             req = Request('GET', url, accepted_errors=[200, 403, 404, 405],
@@ -254,23 +253,8 @@ class RemoteGitProber(Prober):
         return set([RemoteGitControlDirFormat()])
 
 
-if not getattr(Prober, "known_formats", None): # bzr < 2.4
-    from bzrlib.plugins.git.dir import (
-        LocalGitControlDirFormat, BareLocalGitControlDirFormat,
-        )
-    from bzrlib.plugins.git.remote import RemoteGitControlDirFormat
-    ControlDirFormat.register_format(LocalGitControlDirFormat())
-    ControlDirFormat.register_format(BareLocalGitControlDirFormat())
-    ControlDirFormat.register_format(RemoteGitControlDirFormat())
-    # Provide RevisionTree.get_file_revision, so various parts of bzr-svn
-    # can avoid inventories.
-    from bzrlib.revisiontree import RevisionTree
-    def get_file_revision(tree, file_id, path=None):
-        return tree.inventory[file_id].revision
-    RevisionTree.get_file_revision = get_file_revision
-
 ControlDirFormat.register_prober(LocalGitProber)
-ControlDirFormat._server_probers.insert(0, RemoteGitProber)
+ControlDirFormat._server_probers.append(RemoteGitProber)
 
 register_transport_proto('git://',
         help="Access using the Git smart server protocol.")
@@ -316,18 +300,10 @@ def update_stanza(rev, stanza):
     else:
         stanza.add("git-commit", git_commit)
 
-try:
-    from bzrlib.hooks import install_lazy_named_hook
-except ImportError: # Compatibility with bzr < 2.4
-    from bzrlib.version_info_formats.format_rio import (
-        RioVersionInfoBuilder,
-        )
-    RioVersionInfoBuilder.hooks.install_named_hook('revision', update_stanza,
-        "git commits")
-else:
-    install_lazy_named_hook("bzrlib.version_info_formats.format_rio",
-        "RioVersionInfoBuilder.hooks", "revision", update_stanza,
-        "git commits")
+from bzrlib.hooks import install_lazy_named_hook
+install_lazy_named_hook("bzrlib.version_info_formats.format_rio",
+    "RioVersionInfoBuilder.hooks", "revision", update_stanza,
+    "git commits")
 
 
 from bzrlib.transport import transport_server_registry
@@ -339,11 +315,11 @@ transport_server_registry.register_lazy('git',
 transport_server_registry.register_lazy('git-receive-pack',
     'bzrlib.plugins.git.server',
     'serve_git_receive_pack',
-    help='Git Smart server receive pack command (inetd mode only)')
+    help='Git Smart server receive pack command. (inetd mode only)')
 transport_server_registry.register_lazy('git-upload-pack',
     'bzrlib.plugins.git.server',
     'serve_git_upload_pack',
-    help='Git Smart server upload pack command (inetd mode only)')
+    help='Git Smart server upload pack command. (inetd mode only)')
 
 from bzrlib.repository import (
     format_registry as repository_format_registry,
@@ -352,14 +328,10 @@ from bzrlib.repository import (
 repository_network_format_registry.register_lazy('git',
     'bzrlib.plugins.git.repository', 'GitRepositoryFormat')
 
-try:
-    register_extra_lazy_repository_format = getattr(repository_format_registry,
-        "register_extra_lazy")
-except AttributeError: # bzr < 2.4
-    pass
-else:
-    register_extra_lazy_repository_format('bzrlib.plugins.git.repository',
-        'GitRepositoryFormat')
+register_extra_lazy_repository_format = getattr(repository_format_registry,
+    "register_extra_lazy")
+register_extra_lazy_repository_format('bzrlib.plugins.git.repository',
+    'GitRepositoryFormat')
 
 from bzrlib.branch import (
     network_format_registry as branch_network_format_registry,
@@ -367,29 +339,21 @@ from bzrlib.branch import (
 branch_network_format_registry.register_lazy('git',
     'bzrlib.plugins.git.branch', 'GitBranchFormat')
 
-try:
-    from bzrlib.branch import (
-        format_registry as branch_format_registry,
-        )
-except ImportError: # bzr < 2.4
-    pass
-else:
-    branch_format_registry.register_extra_lazy(
-        'bzrlib.plugins.git.branch',
-        'GitBranchFormat',
-        )
+from bzrlib.branch import (
+    format_registry as branch_format_registry,
+    )
+branch_format_registry.register_extra_lazy(
+    'bzrlib.plugins.git.branch',
+    'GitBranchFormat',
+    )
 
-try:
-    from bzrlib.workingtree import (
-        format_registry as workingtree_format_registry,
-        )
-except ImportError: # bzr < 2.4
-    pass
-else:
-    workingtree_format_registry.register_extra_lazy(
-        'bzrlib.plugins.git.workingtree',
-        'GitWorkingTreeFormat',
-        )
+from bzrlib.workingtree import (
+    format_registry as workingtree_format_registry,
+    )
+workingtree_format_registry.register_extra_lazy(
+    'bzrlib.plugins.git.workingtree',
+    'GitWorkingTreeFormat',
+    )
 
 controldir_network_format_registry.register_lazy('git',
     "bzrlib.plugins.git.dir", "GitControlDirFormat")
@@ -423,7 +387,11 @@ def update_git_cache(repository, revid):
     store = BazaarObjectStore(repository)
     store.lock_write()
     try:
-        parent_revisions = set(repository.get_parent_map([revid])[revid])
+        try:
+            parent_revisions = set(repository.get_parent_map([revid])[revid])
+        except KeyError:
+            # Isn't this a bit odd - how can a revision that was just committed be missing?
+            return
         missing_revisions = store._missing_revisions(parent_revisions)
         if not missing_revisions:
             # Only update if the cache was up to date previously
@@ -440,26 +408,44 @@ def post_commit_update_cache(local_branch, master_branch, old_revno, old_revid,
 
 
 def loggerhead_git_hook(branch_app, environ):
-    from bzrlib.config import GlobalConfig
     branch = branch_app.branch
-    if GlobalConfig().get_user_option('http_git') != 'True':
+    config_stack = branch.get_config_stack()
+    if config_stack.get('http_git'):
         return None
     from bzrlib.plugins.git.server import git_http_hook
     return git_http_hook(branch, environ['REQUEST_METHOD'],
         environ['PATH_INFO'])
 
-try:
-    from bzrlib.hooks import install_lazy_named_hook
-except ImportError: # Compatibility with bzr < 2.4
-    pass
-else:
-    install_lazy_named_hook("bzrlib.branch",
-        "Branch.hooks", "post_commit", post_commit_update_cache,
-        "git cache")
-    install_lazy_named_hook("bzrlib.plugins.loggerhead.apps.branch",
-        "BranchWSGIApp.hooks", "controller",
-        loggerhead_git_hook, "git support")
+install_lazy_named_hook("bzrlib.branch",
+    "Branch.hooks", "post_commit", post_commit_update_cache,
+    "git cache")
+install_lazy_named_hook("bzrlib.plugins.loggerhead.apps.branch",
+    "BranchWSGIApp.hooks", "controller",
+    loggerhead_git_hook, "git support")
 
+from bzrlib.directory_service import directories
+
+directories.register_lazy('github:', 'bzrlib.plugins.git.directory',
+                          'GitHubDirectory',
+                          'GitHub directory.')
+directories.register_lazy('git@github.com:', 'bzrlib.plugins.git.directory',
+                          'GitHubDirectory',
+                          'GitHub directory.')
+
+from bzrlib.config import (
+    option_registry,
+    Option,
+    bool_from_store,
+    )
+
+option_registry.register(
+    Option('git.http',
+           default=None, from_unicode=bool_from_store, invalid='warning',
+           help='''\
+Allow fetching of Git packs over HTTP.
+
+This enables support for fetching Git packs over HTTP in Loggerhead.
+'''))
 
 def test_suite():
     from bzrlib.plugins.git import tests
