@@ -16,6 +16,8 @@
 
 """Map from Git sha's to Bazaar objects."""
 
+from __future__ import absolute_import
+
 from dulwich.objects import (
     Blob,
     Commit,
@@ -77,8 +79,12 @@ class LRUTreeCache(object):
 
     def __init__(self, repository):
         def approx_tree_size(tree):
-            # Very rough estimate, 1k per inventory entry
-            return len(tree.inventory) * 1024
+            # Very rough estimate, 250 per inventory entry
+            try:
+                inv = tree.root_inventory
+            except AttributeError:
+                inv = tree.inventory
+            return len(inv) * 250
         self.repository = repository
         self._cache = lru_cache.LRUSizeCache(max_size=MAX_TREE_CACHE_SIZE,
             after_cleanup_size=None, compute_size=approx_tree_size)
@@ -175,80 +181,84 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
     except IndexError:
         base_tree = tree._repository.revision_tree(NULL_REVISION)
         other_parent_trees = []
-    def find_unchanged_parent_ie(ie, parent_trees):
-        assert ie.kind in ("symlink", "file")
+    def find_unchanged_parent_ie(file_id, kind, other, parent_trees):
         for ptree in parent_trees:
             try:
-                pie = ptree.inventory[ie.file_id]
+                pkind = ptree.kind(file_id)
             except errors.NoSuchId:
                 pass
             else:
-                if (pie.text_sha1 == ie.text_sha1 and
-                    pie.kind == ie.kind and
-                    pie.symlink_target == ie.symlink_target):
-                    return pie
+                if kind == "file":
+                    if (pkind == "file" and 
+                        ptree.get_file_sha1(file_id) == other):
+                        return (file_id, ptree.get_file_revision(file_id))
+                if kind == "symlink":
+                    if (pkind == "symlink" and
+                        ptree.get_symlink_target(file_id) == other):
+                        return (file_id, ptree.get_file_revision(file_id))
         raise KeyError
 
     # Find all the changed blobs
     for (file_id, path, changed_content, versioned, parent, name, kind,
          executable) in tree.iter_changes(base_tree):
         if kind[1] == "file":
-            ie = tree.inventory[file_id]
             if changed_content:
                 try:
-                    pie = find_unchanged_parent_ie(ie, other_parent_trees)
+                    (pfile_id, prevision) = find_unchanged_parent_ie(file_id, kind[1], tree.get_file_sha1(file_id), other_parent_trees)
                 except KeyError:
                     pass
                 else:
                     try:
-                        shamap[ie.file_id] = idmap.lookup_blob_id(
-                            pie.file_id, pie.revision)
+                        shamap[file_id] = idmap.lookup_blob_id(
+                            pfile_id, prevision)
                     except KeyError:
                         # no-change merge ?
                         blob = Blob()
-                        blob.data = tree.get_file_text(ie.file_id)
-                        shamap[ie.file_id] = blob.id
+                        blob.data = tree.get_file_text(file_id)
+                        shamap[file_id] = blob.id
             if not file_id in shamap:
-                new_blobs.append((path[1], ie))
+                new_blobs.append((path[1], file_id))
             new_trees[posixpath.dirname(path[1])] = parent[1]
         elif kind[1] == "symlink":
-            ie = tree.inventory[file_id]
             if changed_content:
-                blob = symlink_to_blob(ie)
+                target = tree.get_symlink_target(file_id)
+                blob = symlink_to_blob(target)
                 shamap[file_id] = blob.id
                 try:
-                    find_unchanged_parent_ie(ie, other_parent_trees)
+                    find_unchanged_parent_ie(file_id, kind[1], target, other_parent_trees)
                 except KeyError:
-                    yield path[1], blob, ie
+                    yield path[1], blob, (file_id, tree.get_file_revision(file_id, path[1]))
             new_trees[posixpath.dirname(path[1])] = parent[1]
         elif kind[1] not in (None, "directory"):
             raise AssertionError(kind[1])
         if (path[0] not in (None, "") and
             tree.has_id(parent[0]) and
-            tree.inventory[parent[0]].kind == "directory"):
+            tree.kind(parent[0]) == "directory"):
             # Removal
             new_trees[posixpath.dirname(path[0])] = parent[0]
 
     # Fetch contents of the blobs that were changed
-    for (path, ie), chunks in tree.iter_files_bytes(
-        [(ie.file_id, (path, ie)) for (path, ie) in new_blobs]):
+    for (path, file_id), chunks in tree.iter_files_bytes(
+        [(file_id, (path, file_id)) for (path, file_id) in new_blobs]):
         obj = Blob()
         obj.chunked = chunks
-        yield path, obj, ie
-        shamap[ie.file_id] = obj.id
+        yield path, obj, (file_id, tree.get_file_revision(file_id, path))
+        shamap[file_id] = obj.id
 
     for path in unusual_modes:
         parent_path = posixpath.dirname(path)
-        new_trees[parent_path] = tree.path2id(parent_path)
+        file_id = tree.path2id(parent_path)
+        assert file_id is not None, "Unable to find file id for %r" % parent_path
+        new_trees[parent_path] = file_id
 
     trees = {}
     while new_trees:
         items = new_trees.items()
         new_trees = {}
         for path, file_id in items:
-            parent_id = tree.inventory[file_id].parent_id
-            if parent_id is not None:
+            if path != "":
                 parent_path = urlutils.dirname(path)
+                parent_id = tree.path2id(parent_path)
                 new_trees[parent_path] = parent_id
             trees[path] = file_id
 
@@ -268,22 +278,28 @@ def _tree_to_objects(tree, parent_trees, idmap, unusual_modes,
             elif ie.kind == "directory":
                 # Not all cache backends store the tree information, 
                 # calculate again from scratch
-                ret = directory_to_tree(ie, ie_to_hexsha, unusual_modes,
-                    dummy_file_name)
+                ret = directory_to_tree(ie.children, ie_to_hexsha,
+                    unusual_modes, dummy_file_name, ie.parent_id is None)
                 if ret is None:
                     return ret
                 return ret.id
             else:
                 raise AssertionError
 
+    try:
+        inv = tree.root_inventory
+    except AttributeError:
+        inv = tree.inventory
+
     for path in sorted(trees.keys(), reverse=True):
-        ie = tree.inventory[trees[path]]
-        assert ie.kind == "directory"
-        obj = directory_to_tree(ie, ie_to_hexsha, unusual_modes,
-            dummy_file_name)
+        file_id = trees[path]
+        assert tree.kind(file_id) == 'directory'
+        ie = inv[file_id]
+        obj = directory_to_tree(ie.children, ie_to_hexsha, unusual_modes,
+            dummy_file_name, path == "")
         if obj is not None:
-            yield path, obj, ie
-            shamap[ie.file_id] = obj.id
+            yield path, obj, (file_id, )
+            shamap[file_id] = obj.id
 
 
 class PackTupleIterable(object):
@@ -418,14 +434,14 @@ class BazaarObjectStore(BaseObjectStore):
         parent_trees = self.tree_cache.revision_trees(
             [p for p in rev.parent_ids if p in present_parents])
         root_tree = None
-        for path, obj, ie in _tree_to_objects(tree, parent_trees,
+        for path, obj, bzr_key_data in _tree_to_objects(tree, parent_trees,
                 self._cache.idmap, unusual_modes, self.mapping.BZR_DUMMY_FILE):
             if path == "":
                 root_tree = obj
-                root_ie = ie
+                root_key_data = bzr_key_data
                 # Don't yield just yet
             else:
-                yield path, obj, ie
+                yield path, obj, bzr_key_data
         if root_tree is None:
             # Pointless commit - get the tree sha elsewhere
             if not rev.parent_ids:
@@ -433,14 +449,14 @@ class BazaarObjectStore(BaseObjectStore):
             else:
                 base_sha1 = self._lookup_revision_sha1(rev.parent_ids[0])
                 root_tree = self[self[base_sha1].tree]
-            root_ie = tree.inventory.root
+            root_key_data = (tree.get_root_id(), )
         if not lossy and self.mapping.BZR_FILE_IDS_FILE is not None:
             b = self._create_fileid_map_blob(tree)
             if b is not None:
                 root_tree[self.mapping.BZR_FILE_IDS_FILE] = (
                     (stat.S_IFREG | 0644), b.id)
                 yield self.mapping.BZR_FILE_IDS_FILE, b, None
-        yield "", root_tree, root_ie
+        yield "", root_tree, root_key_data
         if not lossy:
             testament3 = StrictTestament3(rev, tree)
             verifiers = { "testament3-sha1": testament3.as_sha1() }
@@ -487,9 +503,8 @@ class BazaarObjectStore(BaseObjectStore):
             if blob.id != expected_sha and blob.data == "":
                 # Perhaps it's a symlink ?
                 tree = self.tree_cache.revision_tree(revision)
-                entry = tree.inventory[fileid]
-                if entry.kind == 'symlink':
-                    blob = symlink_to_blob(entry)
+                if tree.kind(fileid) == 'symlink':
+                    blob = symlink_to_blob(tree.get_symlink_target(fileid))
             _check_expected_sha(expected_sha, blob)
             yield blob
 
@@ -525,8 +540,13 @@ class BazaarObjectStore(BaseObjectStore):
                 return self._lookup_revision_sha1(entry.reference_revision)
             else:
                 raise AssertionError("unknown entry kind '%s'" % entry.kind)
-        tree = directory_to_tree(bzr_tree.inventory[fileid], get_ie_sha1, unusual_modes,
-            self.mapping.BZR_DUMMY_FILE)
+        try:
+            inv = bzr_tree.root_inventory
+        except AttributeError:
+            inv = bzr_tree.inventory
+        tree = directory_to_tree(inv[fileid].children,
+                get_ie_sha1, unusual_modes, self.mapping.BZR_DUMMY_FILE,
+                bzr_tree.get_root_id() == fileid)
         if (bzr_tree.get_root_id() == fileid and
             self.mapping.BZR_FILE_IDS_FILE is not None):
             if tree is None:
