@@ -22,7 +22,6 @@ from dulwich.repo import (
 
 import os
 
-from bzrlib.branch import Branch
 from bzrlib import (
     version_info as bzrlib_version,
     )
@@ -186,8 +185,9 @@ class TestGitBlackBox(ExternalBase):
         r.do_commit(ref="refs/heads/abranch", committer="Joe <joe@example.com>", message="Dummy")
         r.do_commit(ref="refs/heads/bbranch", committer="Joe <joe@example.com>", message="Dummy")
         self.run_bzr(["git-import", "a", "b"])
-        self.assertEquals(set(["abranch", "bbranch", ".bzr"]),
-                set(os.listdir("b")))
+        self.assertEquals(set([".bzr"]), set(os.listdir("b")))
+        self.assertEquals(set(["abranch", "bbranch"]),
+                set(BzrDir.open("b").get_branches().keys()))
 
     def test_git_import_incremental(self):
         r = GitRepo.init("a", mkdir=True)
@@ -196,7 +196,9 @@ class TestGitBlackBox(ExternalBase):
         r.do_commit(ref="refs/heads/abranch", committer="Joe <joe@example.com>", message="Dummy")
         self.run_bzr(["git-import", "a", "b"])
         self.run_bzr(["git-import", "a", "b"])
-        self.assertEquals(set(["abranch", ".bzr"]), set(os.listdir("b")))
+        self.assertEquals(set([".bzr"]), set(os.listdir("b")))
+        b = BzrDir.open("b")
+        self.assertEquals(["abranch"], b.get_branches().keys())
 
     def test_git_import_tags(self):
         r = GitRepo.init("a", mkdir=True)
@@ -205,13 +207,13 @@ class TestGitBlackBox(ExternalBase):
         cid = r.do_commit(ref="refs/heads/abranch", committer="Joe <joe@example.com>", message="Dummy")
         r["refs/tags/atag"] = cid
         self.run_bzr(["git-import", "a", "b"])
-        self.assertEquals(set(["abranch", ".bzr"]), set(os.listdir("b")))
-        b = Branch.open("b/abranch")
-        self.assertEquals(["atag"], b.tags.get_tag_dict().keys())
+        self.assertEquals(set([".bzr"]), set(os.listdir("b")))
+        b = BzrDir.open("b")
+        self.assertEquals(["abranch"], b.get_branches().keys())
+        self.assertEquals(["atag"],
+                b.open_branch("abranch").tags.get_tag_dict().keys())
 
     def test_git_import_colo(self):
-        if bzrlib_version < (2, 5, 0):
-            raise TestSkipped("colocated branches not supported with bzr < 2.5")
         r = GitRepo.init("a", mkdir=True)
         self.build_tree(["a/file"])
         r.stage("file")
@@ -245,3 +247,24 @@ class TestGitBlackBox(ExternalBase):
         self.assertEquals(stderr, "")
         self.assertTrue("refs/tags/atag -> " in stdout)
         self.assertTrue("HEAD -> " in stdout)
+
+    def test_dpush(self):
+        r = GitRepo.init("gitr", mkdir=True)
+        self.build_tree_contents([("gitr/foo", "hello from git")])
+        r.stage("foo")
+        r.do_commit("message", committer="Somebody <user@example.com>")
+        self.run_bzr(["branch", "gitr", "bzrb"])
+        self.build_tree_contents([("bzrb/foo", "hello from bzr")])
+        self.run_bzr(["commit", "-m", "msg", "bzrb"])
+        self.run_bzr(["dpush", "-d", "bzrb", "gitr"])
+
+    def test_dpush_from_bound(self):
+        r = GitRepo.init("gitr", mkdir=True)
+        self.build_tree_contents([("gitr/foo", "hello from git")])
+        r.stage("foo")
+        r.do_commit("message", committer="Somebody <user@example.com>")
+        self.run_bzr(["branch", "gitr", "bzrm"])
+        self.run_bzr(["checkout", "bzrm", "bzrb"])
+        self.build_tree_contents([("bzrb/foo", "hello from bzr")])
+        self.run_bzr(["commit", "-m", "msg", "bzrb"])
+        self.run_bzr(["dpush", "-d", "bzrb", "gitr"])
