@@ -17,6 +17,8 @@
 
 """Git Trees."""
 
+from __future__ import absolute_import
+
 from dulwich.object_store import tree_lookup_path
 import stat
 import posixpath
@@ -76,6 +78,9 @@ class GitRevisionTree(revisiontree.RevisionTree):
             return None
         return self._fileid_map.lookup_file_id(path.encode('utf-8'))
 
+    def all_file_ids(self):
+        return set(self._fileid_map.all_file_ids())
+
     def get_root_id(self):
         return self.path2id("")
 
@@ -235,6 +240,22 @@ class GitRevisionTree(revisiontree.RevisionTree):
             return None, False, None
         return entry.kind, entry.executable, None
 
+    def path_content_summary(self, path):
+        """See Tree.path_content_summary."""
+        try:
+            (mode, hexsha) = tree_lookup_path(self.store.__getitem__, self.tree, path)
+        except KeyError:
+            return ('missing', None, None, None)
+        kind = mode_kind(mode)
+        if kind == 'file':
+            executable = mode_is_executable(mode)
+            contents = self.store[hexsha].data
+            return (kind, len(contents), executable, osutils.sha_string(contents))
+        elif kind == 'symlink':
+            return (kind, None, None, self.store[hexsha].data)
+        else:
+            return (kind, None, None, None)
+
 
 def tree_delta_from_git_changes(changes, mapping,
         (old_fileid_map, new_fileid_map), specific_file=None,
@@ -292,6 +313,7 @@ def changes_from_git_changes(changes, mapping, specific_file=None,
             oldparent = None
         else:
             oldpath = oldpath.decode("utf-8")
+            assert oldmode is not None
             oldexe = mode_is_executable(oldmode)
             oldkind = mode_kind(oldmode)
             try:
@@ -309,8 +331,13 @@ def changes_from_git_changes(changes, mapping, specific_file=None,
             newparent = None
         else:
             newpath = newpath.decode("utf-8")
-            newexe = mode_is_executable(newmode)
-            newkind = mode_kind(newmode)
+            assert newmode is not None
+            if newmode is not None:
+                newexe = mode_is_executable(newmode)
+                newkind = mode_kind(newmode)
+            else:
+                newexe = False
+                newkind = None
             try:
                 newparentpath, newname = newpath.rsplit("/", 1)
             except ValueError:
